@@ -1,16 +1,19 @@
-# Copyright 1999-2011 Gentoo Foundation
+# Copyright 1999-2014 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
 
 from __future__ import print_function
 
 import logging
+import signal
 import sys
 import textwrap
 import portage
 from portage import os
+from portage.dbapi._expand_new_virt import expand_new_virt
 from portage.output import bold, colorize, darkgreen, green
 from portage._sets import SETPREFIX
-from portage.util import cmp_sort_key
+from portage._sets.base import EditablePackageSet
+from portage.versions import cpv_sort_key, _pkg_str
 
 from _emerge.emergelog import emergelog
 from _emerge.Package import Package
@@ -18,21 +21,20 @@ from _emerge.UninstallFailure import UninstallFailure
 from _emerge.userquery import userquery
 from _emerge.countdown import countdown
 
-def unmerge(root_config, myopts, unmerge_action,
-	unmerge_files, ldpath_mtimes, autoclean=0,
-	clean_world=1, clean_delay=1, ordered=0, raise_on_error=0,
-	scheduler=None, writemsg_level=portage.util.writemsg_level):
+def _unmerge_display(root_config, myopts, unmerge_action,
+	unmerge_files, clean_delay=1, ordered=0,
+	writemsg_level=portage.util.writemsg_level):
+	"""
+	Returns a tuple of (returncode, pkgmap) where returncode is
+	os.EX_OK if no errors occur, and 1 otherwise.
+	"""
 
-	if clean_world:
-		clean_world = myopts.get('--deselect') != 'n'
 	quiet = "--quiet" in myopts
-	enter_invalid = '--ask-enter-invalid' in myopts
 	settings = root_config.settings
 	sets = root_config.sets
 	vartree = root_config.trees["vartree"]
 	candidate_catpkgs=[]
 	global_unmerge=0
-	xterm_titles = "notitles" not in settings.features
 	out = portage.output.EOutput()
 	pkg_cache = {}
 	db_keys = list(vartree.dbapi._aux_cache_keys)
@@ -56,12 +58,28 @@ def unmerge(root_config, myopts, unmerge_action,
 	vdb_lock = None
 	try:
 		if os.access(vdb_path, os.W_OK):
-			vdb_lock = portage.locks.lockdir(vdb_path)
-		realsyslist = sets["system"].getAtoms()
+			vartree.dbapi.lock()
+			vdb_lock = True
+
+		realsyslist = []
+		sys_virt_map = {}
+		for x in sets["system"].getAtoms():
+			for atom in expand_new_virt(vartree.dbapi, x):
+				if not atom.blocker:
+					realsyslist.append(atom)
+					if atom.cp != x.cp:
+						sys_virt_map[atom.cp] = x.cp
+
 		syslist = []
 		for x in realsyslist:
-			mycp = portage.dep_getkey(x)
-			if mycp in settings.getvirtuals():
+			mycp = x.cp
+			# Since Gentoo stopped using old-style virtuals in
+			# 2011, typically it's possible to avoid getvirtuals()
+			# calls entirely. It will not be triggered here by
+			# new-style virtuals since those are expanded to
+			# non-virtual atoms above by expand_new_virt().
+			if mycp.startswith("virtual/") and \
+				mycp in settings.getvirtuals():
 				providers = []
 				for provider in settings.getvirtuals()[mycp]:
 					if vartree.dbapi.match(provider):
@@ -70,15 +88,14 @@ def unmerge(root_config, myopts, unmerge_action,
 					syslist.extend(providers)
 			else:
 				syslist.append(mycp)
-	
-		mysettings = portage.config(clone=settings)
+		syslist = frozenset(syslist)
 	
 		if not unmerge_files:
 			if unmerge_action == "unmerge":
 				print()
 				print(bold("emerge unmerge") + " can only be used with specific package names")
 				print()
-				return 0
+				return 1, {}
 			else:
 				global_unmerge = 1
 	
@@ -92,7 +109,7 @@ def unmerge(root_config, myopts, unmerge_action,
 			#we've got command-line arguments
 			if not unmerge_files:
 				print("\nNo packages to unmerge have been provided.\n")
-				return 0
+				return 1, {}
 			for x in unmerge_files:
 				arg_parts = x.split('/')
 				if x[0] not in [".","/"] and \
@@ -108,7 +125,7 @@ def unmerge(root_config, myopts, unmerge_action,
 					# ebuild and we're in "unmerge" mode, so it's ok.
 					if not os.path.exists(x):
 						print("\n!!! The path '"+x+"' doesn't exist.\n")
-						return 0
+						return 1, {}
 	
 					absx   = os.path.abspath(x)
 					sp_absx = absx.split("/")
@@ -119,14 +136,13 @@ def unmerge(root_config, myopts, unmerge_action,
 					sp_absx_len = len(sp_absx)
 	
 					vdb_path = os.path.join(settings["EROOT"], portage.VDB_PATH)
-					vdb_len  = len(vdb_path)
 	
 					sp_vdb     = vdb_path.split("/")
 					sp_vdb_len = len(sp_vdb)
 	
 					if not os.path.exists(absx+"/CONTENTS"):
 						print("!!! Not a valid db dir: "+str(absx))
-						return 0
+						return 1, {}
 	
 					if sp_absx_len <= sp_vdb_len:
 						# The Path is shorter... so it can't be inside the vdb.
@@ -134,7 +150,7 @@ def unmerge(root_config, myopts, unmerge_action,
 						print(absx)
 						print("\n!!!",x,"cannot be inside "+ \
 							vdb_path+"; aborting.\n")
-						return 0
+						return 1, {}
 	
 					for idx in range(0,sp_vdb_len):
 						if idx >= sp_absx_len or sp_vdb[idx] != sp_absx[idx]:
@@ -142,7 +158,7 @@ def unmerge(root_config, myopts, unmerge_action,
 							print(absx)
 							print("\n!!!", x, "is not inside "+\
 								vdb_path+"; aborting.\n")
-							return 0
+							return 1, {}
 	
 					print("="+"/".join(sp_absx[sp_vdb_len:]))
 					candidate_catpkgs.append(
@@ -269,20 +285,18 @@ def unmerge(root_config, myopts, unmerge_action,
 		numselected = len(all_selected)
 		if global_unmerge and not numselected:
 			portage.writemsg_stdout("\n>>> No outdated packages were found on your system.\n")
-			return 0
+			return 1, {}
 	
 		if not numselected:
 			portage.writemsg_stdout(
 				"\n>>> No packages selected for removal by " + \
 				unmerge_action + "\n")
-			return 0
+			return 1, {}
 	finally:
 		if vdb_lock:
 			vartree.dbapi.flush_cache()
-			portage.locks.unlockdir(vdb_lock)
-	
-	from portage._sets.base import EditablePackageSet
-	
+			vartree.dbapi.unlock()
+
 	# generate a list of package sets that are directly or indirectly listed in "selected",
 	# as there is no persistent list of "installed" sets
 	installed_sets = ["selected"]
@@ -314,19 +328,24 @@ def unmerge(root_config, myopts, unmerge_action,
 				# by a concurrent process.
 				continue
 
-			if unmerge_action != "clean" and \
-				root_config.root == "/" and \
-				portage.match_from_list(
-				portage.const.PORTAGE_PACKAGE_ATOM, [pkg]):
-				msg = ("Not unmerging package %s since there is no valid " + \
-				"reason for portage to unmerge itself.") % (pkg.cpv,)
-				for line in textwrap.wrap(msg, 75):
-					out.eerror(line)
-				# adjust pkgmap so the display output is correct
-				pkgmap[cp]["selected"].remove(cpv)
-				all_selected.remove(cpv)
-				pkgmap[cp]["protected"].add(cpv)
-				continue
+			if unmerge_action != "clean" and root_config.root == "/":
+				skip_pkg = False
+				if portage.match_from_list(portage.const.PORTAGE_PACKAGE_ATOM, [pkg]):
+					msg = ("Not unmerging package %s since there is no valid reason "
+						"for Portage to unmerge itself.") % (pkg.cpv,)
+					skip_pkg = True
+				elif vartree.dbapi._dblink(cpv).isowner(portage._python_interpreter):
+					msg = ("Not unmerging package %s since there is no valid reason "
+						"for Portage to unmerge currently used Python interpreter.") % (pkg.cpv,)
+					skip_pkg = True
+				if skip_pkg:
+					for line in textwrap.wrap(msg, 75):
+						out.eerror(line)
+					# adjust pkgmap so the display output is correct
+					pkgmap[cp]["selected"].remove(cpv)
+					all_selected.remove(cpv)
+					pkgmap[cp]["protected"].add(cpv)
+					continue
 
 			parents = []
 			for s in installed_sets:
@@ -388,7 +407,7 @@ def unmerge(root_config, myopts, unmerge_action,
 		writemsg_level(
 			"\n>>> No packages selected for removal by " + \
 			unmerge_action + "\n")
-		return 0
+		return 1, {}
 
 	# Unmerge order only matters in some cases
 	if not ordered:
@@ -427,15 +446,17 @@ def unmerge(root_config, myopts, unmerge_action,
 			#avoid cluttering the preview printout with stuff that isn't getting unmerged
 			continue
 		if not (pkgmap[x]["protected"] or pkgmap[x]["omitted"]) and cp in syslist:
+			virt_cp = sys_virt_map.get(cp)
+			if virt_cp is None:
+				cp_info = "'%s'" % (cp,)
+			else:
+				cp_info = "'%s' (%s)" % (cp, virt_cp)
 			writemsg_level(colorize("BAD","\n\n!!! " + \
-				"'%s' is part of your system profile.\n" % cp),
+				"%s is part of your system profile.\n" % (cp_info,)),
 				level=logging.WARNING, noiselevel=-1)
 			writemsg_level(colorize("WARN","!!! Unmerging it may " + \
 				"be damaging to your system.\n\n"),
 				level=logging.WARNING, noiselevel=-1)
-			if clean_delay and "--pretend" not in myopts and "--ask" not in myopts:
-				countdown(int(settings["EMERGE_WARNING_DELAY"]),
-					colorize("UNMERGE_WARN", "Press Ctrl-C to Stop"))
 		if not quiet:
 			writemsg_level("\n %s\n" % (bold(cp),), noiselevel=-1)
 		else:
@@ -444,20 +465,22 @@ def unmerge(root_config, myopts, unmerge_action,
 			if not quiet:
 				writemsg_level((mytype + ": ").rjust(14), noiselevel=-1)
 			if pkgmap[x][mytype]:
-				sorted_pkgs = [portage.catpkgsplit(mypkg)[1:] for mypkg in pkgmap[x][mytype]]
-				sorted_pkgs.sort(key=cmp_sort_key(portage.pkgcmp))
-				for pn, ver, rev in sorted_pkgs:
-					if rev == "r0":
-						myversion = ver
-					else:
-						myversion = ver + "-" + rev
+				sorted_pkgs = []
+				for mypkg in pkgmap[x][mytype]:
+					try:
+						sorted_pkgs.append(mypkg.cpv)
+					except AttributeError:
+						sorted_pkgs.append(_pkg_str(mypkg))
+				sorted_pkgs.sort(key=cpv_sort_key())
+				for mypkg in sorted_pkgs:
 					if mytype == "selected":
 						writemsg_level(
-							colorize("UNMERGE_WARN", myversion + " "),
+							colorize("UNMERGE_WARN", mypkg.version + " "),
 							noiselevel=-1)
 					else:
 						writemsg_level(
-							colorize("GOOD", myversion + " "), noiselevel=-1)
+							colorize("GOOD", mypkg.version + " "),
+							noiselevel=-1)
 			else:
 				writemsg_level("none ", noiselevel=-1)
 			if not quiet:
@@ -473,9 +496,38 @@ def unmerge(root_config, myopts, unmerge_action,
 			" and " + colorize("GOOD", "'omitted'") + \
 			" packages will not be removed.\n\n")
 
+	return os.EX_OK, pkgmap
+
+def unmerge(root_config, myopts, unmerge_action,
+	unmerge_files, ldpath_mtimes, autoclean=0,
+	clean_world=1, clean_delay=1, ordered=0, raise_on_error=0,
+	scheduler=None, writemsg_level=portage.util.writemsg_level):
+	"""
+	Returns os.EX_OK if no errors occur, 1 if an error occurs, and
+	130 if interrupted due to a 'no' answer for --ask.
+	"""
+
+	if clean_world:
+		clean_world = myopts.get('--deselect') != 'n'
+
+	rval, pkgmap = _unmerge_display(root_config, myopts,
+		unmerge_action, unmerge_files,
+		clean_delay=clean_delay, ordered=ordered,
+		writemsg_level=writemsg_level)
+
+	if rval != os.EX_OK:
+		return rval
+
+	enter_invalid = '--ask-enter-invalid' in myopts
+	vartree = root_config.trees["vartree"]
+	sets = root_config.sets
+	settings = root_config.settings
+	mysettings = portage.config(clone=settings)
+	xterm_titles = "notitles" not in settings.features
+
 	if "--pretend" in myopts:
 		#we're done... return
-		return 0
+		return os.EX_OK
 	if "--ask" in myopts:
 		if userquery("Would you like to unmerge these packages?",
 			enter_invalid) == "No":
@@ -484,19 +536,32 @@ def unmerge(root_config, myopts, unmerge_action,
 			print()
 			print("Quitting.")
 			print()
-			return 0
+			return 128 + signal.SIGINT
 	#the real unmerging begins, after a short delay....
 	if clean_delay and not autoclean:
 		countdown(int(settings["CLEAN_DELAY"]), ">>> Unmerging")
 
+	all_selected = set()
+	all_selected.update(*[x["selected"] for x in pkgmap])
+
+	# Set counter variables
+	curval = 1
+	maxval = len(all_selected)
+
 	for x in range(len(pkgmap)):
 		for y in pkgmap[x]["selected"]:
-			writemsg_level(">>> Unmerging "+y+"...\n", noiselevel=-1)
 			emergelog(xterm_titles, "=== Unmerging... ("+y+")")
+			message = ">>> Unmerging ({0} of {1}) {2}...\n".format(
+				colorize("MERGE_LIST_PROGRESS", str(curval)),
+				colorize("MERGE_LIST_PROGRESS", str(maxval)),
+				y)
+			writemsg_level(message, noiselevel=-1)
+			curval += 1
+
 			mysplit = y.split("/")
 			#unmerge...
-			retval = portage.unmerge(mysplit[0], mysplit[1], settings["ROOT"],
-				mysettings, unmerge_action not in ["clean","prune"],
+			retval = portage.unmerge(mysplit[0], mysplit[1],
+				settings=mysettings,
 				vartree=vartree, ldpath_mtimes=ldpath_mtimes,
 				scheduler=scheduler)
 
@@ -523,5 +588,5 @@ def unmerge(root_config, myopts, unmerge_action,
 			sets["selected"].remove(SETPREFIX + s)
 		sets["selected"].unlock()
 
-	return 1
+	return os.EX_OK
 
