@@ -1,13 +1,18 @@
 # localization.py -- Code to manage/help portage localization.
-# Copyright 2004 Gentoo Foundation
+# Copyright 2004-2014 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Id: localization.py 14290 2009-09-20 11:08:30Z arfrever $
 
+from portage import _unicode_decode
 
 # We define this to make the transition easier for us.
 def _(mystr):
-	return mystr
-
+	"""
+	Always returns unicode, regardless of the input type. This is
+	helpful for avoiding UnicodeDecodeError from __str__() with
+	Python 2, by ensuring that string format operations invoke
+	__unicode__() instead of __str__().
+	"""
+	return _unicode_decode(mystr)
 
 def localization_example():
 	# Dict references allow translators to rearrange word order.
@@ -16,6 +21,7 @@ def localization_example():
 
 	a_value = "value.of.a"
 	b_value = 123
-	c_value = [1,2,3,4]
-	print(_("A: %(a)s -- B: %(b)s -- C: %(c)s") % {"a":a_value,"b":b_value,"c":c_value})
+	c_value = [1, 2, 3, 4]
+	print(_("A: %(a)s -- B: %(b)s -- C: %(c)s") %
+	      {"a": a_value, "b": b_value, "c": c_value})
 
