@@ -1,5 +1,5 @@
 # repoman: Checks
-# Copyright 2007-2013 Gentoo Foundation
+# Copyright 2007-2014 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
 
 """This module contains functions used in Repoman to ascertain the quality
@@ -15,8 +15,7 @@ import repoman.errors as errors
 import portage
 from portage.eapi import eapi_supports_prefix, eapi_has_implicit_rdepend, \
 	eapi_has_src_prepare_and_src_configure, eapi_has_dosed_dohard, \
-	eapi_exports_AA
-from portage.const import _ENABLE_INHERIT_CHECK
+	eapi_exports_AA, eapi_has_pkg_pretend
 
 class LineCheck(object):
 	"""Run a check on a line of an ebuild."""
@@ -387,6 +386,7 @@ class InheritDeprecated(LineCheck):
 		"distutils": "distutils-r1",
 		"gems": "ruby-fakegem",
 		"git": "git-2",
+		"mono": "mono-env",
 		"mozconfig-2": "mozconfig-3",
 		"mozcoreconf": "mozcoreconf-2",
 		"php-ext-pecl-r1": "php-ext-pecl-r2",
@@ -634,29 +634,6 @@ _eclass_info = {
 	}
 }
 
-if not _ENABLE_INHERIT_CHECK:
-	# Since the InheritEclass check is experimental, in the stable branch
-	# we emulate the old eprefixify.defined and inherit.autotools checks.
-	_eclass_info = {
-		'autotools': {
-			'funcs': (
-				'eaclocal', 'eautoconf', 'eautoheader',
-				'eautomake', 'eautoreconf', '_elibtoolize',
-				'eautopoint'
-			),
-			'comprehensive': True,
-			'ignore_missing': True,
-			'exempt_eclasses': ('git', 'git-2', 'subversion', 'autotools-utils')
-		},
-
-		'prefix': {
-			'funcs': (
-				'eprefixify',
-			),
-			'comprehensive': False
-		}
-	}
-
 class EMakeParallelDisabled(PhaseCheck):
 	"""Check for emake -j1 calls which disable parallelization."""
 	repoman_check_name = 'upstream.workaround'
@@ -681,8 +658,8 @@ class NoAsNeeded(LineCheck):
 	error = errors.NO_AS_NEEDED
 
 class PreserveOldLib(LineCheck):
-	"""Check for calls to the preserve_old_lib function."""
-	repoman_check_name = 'upstream.workaround'
+	"""Check for calls to the deprecated preserve_old_lib function."""
+	repoman_check_name = 'ebuild.minorsyn'
 	re = re.compile(r'.*preserve_old_lib')
 	error = errors.PRESERVE_OLD_LIB
 
@@ -754,6 +731,21 @@ class DeprecatedHasq(LineCheck):
 	re = re.compile(r'(^|.*\b)hasq\b')
 	error = errors.HASQ_ERROR
 
+# EAPI <2 checks
+class UndefinedSrcPrepareSrcConfigurePhases(LineCheck):
+	repoman_check_name = 'EAPI.incompatible'
+	src_configprepare_re = re.compile(r'\s*(src_configure|src_prepare)\s*\(\)')
+
+	def check_eapi(self, eapi):
+		return not eapi_has_src_prepare_and_src_configure(eapi)
+
+	def check(self, num, line):
+		m = self.src_configprepare_re.match(line)
+		if m is not None:
+			return ("'%s'" % m.group(1)) + \
+				" phase is not defined in EAPI < 2 on line: %d"
+
+
 # EAPI-3 checks
 class Eapi3DeprecatedFuncs(LineCheck):
 	repoman_check_name = 'EAPI.deprecated'
@@ -768,6 +760,20 @@ class Eapi3DeprecatedFuncs(LineCheck):
 			return ("'%s'" % m.group(1)) + \
 				" has been deprecated in EAPI=3 on line: %d"
 
+# EAPI <4 checks
+class UndefinedPkgPretendPhase(LineCheck):
+	repoman_check_name = 'EAPI.incompatible'
+	pkg_pretend_re = re.compile(r'\s*(pkg_pretend)\s*\(\)')
+
+	def check_eapi(self, eapi):
+		return not eapi_has_pkg_pretend(eapi)
+
+	def check(self, num, line):
+		m = self.pkg_pretend_re.match(line)
+		if m is not None:
+			return ("'%s'" % m.group(1)) + \
+				" phase is not defined in EAPI < 4 on line: %d"
+
 # EAPI-4 checks
 class Eapi4IncompatibleFuncs(LineCheck):
 	repoman_check_name = 'EAPI.incompatible'
@@ -823,10 +829,39 @@ class PortageInternalVariableAssignment(LineCheck):
 		return e
 
 _base_check_classes = (InheritEclass, LineCheck, PhaseCheck)
-_constant_checks = tuple(chain((v() for k, v in globals().items()
-	if isinstance(v, type) and issubclass(v, LineCheck) and v not in _base_check_classes),
-	(InheritEclass(k, **portage._native_kwargs(kwargs))
-	for k, kwargs in _eclass_info.items())))
+_constant_checks = None
+
+def _init(experimental_inherit=False):
+
+	global _constant_checks, _eclass_info
+
+	if not experimental_inherit:
+		# Emulate the old eprefixify.defined and inherit.autotools checks.
+		_eclass_info = {
+			'autotools': {
+				'funcs': (
+					'eaclocal', 'eautoconf', 'eautoheader',
+					'eautomake', 'eautoreconf', '_elibtoolize',
+					'eautopoint'
+				),
+				'comprehensive': True,
+				'ignore_missing': True,
+				'exempt_eclasses': ('git', 'git-2', 'subversion', 'autotools-utils')
+			},
+
+			'prefix': {
+				'funcs': (
+					'eprefixify',
+				),
+				'comprehensive': False
+			}
+		}
+
+	_constant_checks = tuple(chain((v() for k, v in globals().items()
+		if isinstance(v, type) and issubclass(v, LineCheck) and
+		v not in _base_check_classes),
+		(InheritEclass(k, **portage._native_kwargs(kwargs))
+		for k, kwargs in _eclass_info.items())))
 
 _here_doc_re = re.compile(r'.*\s<<[-]?(\w+)$')
 _ignore_comment_re = re.compile(r'^\s*#')
@@ -834,6 +869,8 @@ _ignore_comment_re = re.compile(r'^\s*#')
 def run_checks(contents, pkg):
 	unicode_escape_codec = codecs.lookup('unicode_escape')
 	unicode_escape = lambda x: unicode_escape_codec.decode(x)[0]
+	if _constant_checks is None:
+		_init()
 	checks = _constant_checks
 	here_doc_delim = None
 	multiline = None
